@@ -366,13 +366,14 @@ load_module_elements(JSContext *context,
         }
 
         while (!JSID_IS_VOID(idp)) {
-            const char *name;
+            char *name;
 
             if (!gjs_get_string_id(context, idp, &name)) {
                 continue;
             }
 
-            g_ptr_array_add(iter->elements, g_strdup(name));
+            /* Pass ownership of name */
+            g_ptr_array_add(iter->elements, name);
 
             if (!JS_NextProperty(context, jsiter, &idp)) {
                 break;
@@ -513,6 +514,15 @@ do_import(JSContext  *context,
     full_path = NULL;
     directories = NULL;
 
+    /* First try importing an internal module like byteArray */
+    if (gjs_is_registered_native_module(context, obj, name) &&
+        import_native_file(context, obj, name, NULL)) {
+        gjs_debug(GJS_DEBUG_IMPORTER,
+                  "successfully imported module '%s'", name);
+        result = JS_TRUE;
+        goto out;
+    }
+
     for (i = 0; i < search_path_len; ++i) {
         jsval elem;
 
@@ -538,6 +548,10 @@ do_import(JSContext  *context,
         if (!gjs_string_to_utf8(context, elem, &dirname))
             goto out; /* Error message already set */
 
+        /* Ignore empty path elements */
+        if (dirname[0] == '\0')
+            continue;
+
         /* Try importing __init__.js and loading the symbol from it */
         if (full_path)
             g_free(full_path);
@@ -563,14 +577,6 @@ do_import(JSContext  *context,
             }
         }
 
-        /* First try importing an internal module like byteArray */
-        if (import_native_file(context, obj, name, NULL)) {
-            gjs_debug(GJS_DEBUG_IMPORTER,
-                      "successfully imported module '%s'", name);
-            result = JS_TRUE;
-            goto out;
-        }
-
         /* Second try importing a directory (a sub-importer) */
         if (full_path)
             g_free(full_path);
@@ -918,7 +924,8 @@ importer_new_resolve(JSContext *context,
                      JSObject **objp)
 {
     Importer *priv;
-    const char *name;
+    char *name;
+    JSBool ret = JS_TRUE;
 
     *objp = NULL;
 
@@ -929,23 +936,25 @@ importer_new_resolve(JSContext *context,
     if (strcmp(name, "valueOf") == 0 ||
         strcmp(name, "toString") == 0 ||
         strcmp(name, "__iterator__") == 0)
-        return JS_TRUE;
+        goto out;
 
     priv = priv_from_js(context, obj);
     gjs_debug_jsprop(GJS_DEBUG_IMPORTER, "Resolve prop '%s' hook obj %p priv %p", name, obj, priv);
 
-    if (priv == NULL)
-        return JS_TRUE; /* we are the prototype, or have the wrong class */
+    if (priv == NULL) /* we are the prototype, or have the wrong class */
+        goto out;
 
     JS_BeginRequest(context);
     if (do_import(context, obj, priv, name)) {
         *objp = obj;
-        JS_EndRequest(context);
-        return JS_TRUE;
     } else {
-        JS_EndRequest(context);
-        return JS_FALSE;
+        ret = JS_FALSE;
     }
+    JS_EndRequest(context);
+
+ out:
+    g_free(name);
+    return ret;
 }
 
 /* If we set JSCLASS_CONSTRUCT_PROTOTYPE flag, then this is called on
